/************************************************************
*
* REXLANG - Using the TSL2561 Lux Sensor via I2C
* Calculates lux using the T, FN, or CL packages
*
*************************************************************/

#define I2CDEV_FNAME 67 // I2C device is defined by the fname parameter of the REXLANG block (e.g. set it to /dev/i2c-1 on the Raspberry Pi minicomputer)

//assigning inputs to variables, these variables are READ-ONLY
//long input(3) digital_out; //the signal controlling the outputs is connected to input u3 of the REXLANG block
double output(0) output_y0; //Testing parameters. This will later be used to set gains and integration times
double output(10) lux; //the state of the input pins is published via output y10 of the REXLANG block 

double parameter(0) param_p0; //Testing parameters. This will later be used to set gains and integration times

//declaration of variables
long i2c_bufTx[3]; //buffer for transmitting data
long i2c_bufRx[3]; //buffer for receiving data
long i2c_bus_handle;
long i2c_chip_address;
long i2c_write_count;
long i2c_read_count;
long i2c_ret_fun;

long dataZeroL;
long dataZeroH;
long dataOneL;
long dataOneH;

double channelZero;
double channelOne;

//the init procedure is executed once when the REXLANG function block initializes
long init(void)
{
    i2c_bus_handle = Open(I2CDEV_FNAME); // open I2C bus
    i2c_chip_address = 0x39; // 7-bit address of the TSL2561 I2C device
    
    //Turning on the Sensor
    i2c_bufTx[0] = 0x00; //Control Register 
    i2c_bufTx[1] = 0x03; //Power Up
    i2c_write_count = 2;
    i2c_read_count = 0;
    i2c_ret_fun = I2C(i2c_bus_handle, i2c_chip_address, i2c_bufTx, i2c_write_count, i2c_bufRx, i2c_read_count);

    //Possible gain, and integration time adjustments here

    //Possible Interrupt adjustements here

    return 0;
}

//the main procedure is executed once in each sampling period
long main(void)
{
    i2c_bufTx[0] = 0x8C; //read Ch. 0 Low byte   
    i2c_write_count = 1;
    i2c_read_count = 1;
    i2c_ret_fun = I2C(i2c_bus_handle, i2c_chip_address, i2c_bufTx, i2c_write_count, i2c_bufRx, i2c_read_count);
    dataZeroL = i2c_bufRx[0];

    i2c_bufTx[1] = 0x8D; //read Ch. 0 High byte
    i2c_write_count = 1;
    i2c_read_count = 1;
    i2c_ret_fun = I2C(i2c_bus_handle, i2c_chip_address, i2c_bufTx, i2c_write_count, i2c_bufRx, i2c_read_count);
    dataZeroH = i2c_bufRx[0];

    i2c_bufTx[1] = 0x8E; //read Ch. 1 Low byte
    i2c_write_count = 1;
    i2c_read_count = 1;
    i2c_ret_fun = I2C(i2c_bus_handle, i2c_chip_address, i2c_bufTx, i2c_write_count, i2c_bufRx, i2c_read_count);
    dataOneH = i2c_bufRx[0];

    i2c_bufTx[1] = 0x8F; //read Ch. 1 High byte
    i2c_write_count = 1;
    i2c_read_count = 1;
    i2c_ret_fun = I2C(i2c_bus_handle, i2c_chip_address, i2c_bufTx, i2c_write_count, i2c_bufRx, i2c_read_count);
    dataOneH = i2c_bufRx[0];

    channelZero = dataZeroL + 256 * dataZeroH;
    channelOne = dataOneL + 256 * dataOneH;

//calculating lux
if (channelZero > 0){
    if (0 < channelOne/channelZero && channelOne/channelZero <= 0.50)
	lux = 0.0304 * channelZero - 0.062 * channelZero * pow((channelOne/channelZero),1.4);
    else if (0.50 < channelOne/channelZero && channelOne/channelZero <= 0.61)
	lux = 0.0224 * channelZero - 0.031 * channelOne;
    else if (0.61 < channelOne/channelZero && channelOne/channelZero <= 0.80)
	lux = 0.0128 * channelZero - 0.0153 * channelOne;
    else if (0.80 < channelOne/channelZero && channelOne/channelZero <= 1.30)
	lux = 0.00146 * channelZero - 0.00112 * channelOne;
    else
	lux = 0;
}

    output_y0 = param_p0;

    return 0;
}

//the exit procedure is executed once when the task is correctly terminated
// (system shutdown, downloading new control algorithm, etc.)
long exit(void)
{

  //Maybe send a power down signal to the tsl2561 here
  if(i2c_bus_handle>=0) Close(i2c_bus_handle); // close I2C bus
  return 0;
}